/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2002 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */
package org.objectweb.medor.expression;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.Operator;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.ExpressionPrinter;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/*
 * @author S.Chassande-Barrioz
 */

public class TestExpressionHelper extends TestLogHelper {

	public TestExpressionHelper(String testName) {
		super(testName);
	};

	public static void equals(String msg, Expression e1, Expression e2, Logger log)
			throws ExpressionException {
		log.log(BasicLevel.DEBUG, "Comparing expression "
			+ ExpressionPrinter.e2str(e1) + " to "
			+ ExpressionPrinter.e2str(e2));
		if (e1==null && e2==null)
			return;
		if (e1==null)
			fail(msg + " The e1 is null and e2=" + ExpressionPrinter.e2str(e2));
		if (e2==null)
			fail(msg + " The e2 is null and e1=" + ExpressionPrinter.e2str(e1));
		log.log(BasicLevel.DEBUG, e1.getClass().getName());
		if (e1.getClass() != e2.getClass()) {
			fail(msg + "Not same class: \n"
				+ "e1=" + e1.getClass().getName() + ": "
				+ ExpressionPrinter.e2str(e1) + "\n"
				+ "e2=" + e2.getClass().getName() + ": "
				+ ExpressionPrinter.e2str(e2));
		}
		if (e1 instanceof Operator) {
			for(int i=0; i<((Operator) e1).getOperandNumber(); i++) {
				equals(msg + " " + ((Operator)e1).getOperatorString() + "." + i,
				((Operator) e1).getExpression(i),
				((Operator) e2).getExpression(i),
					log);
			}
		}
		else if (e1 instanceof BasicOperand) {
			BasicOperand bo1 = (BasicOperand) e1;
			BasicOperand bo2 = (BasicOperand) e2;
			assertEquals(msg + " Basic operand does not have the same type",
				bo1.getType().getTypeCode(),bo2.getType().getTypeCode());
			log.log(BasicLevel.DEBUG, "type=" + bo1.getType().getJavaName());
			try {
				switch(bo1.getType().getTypeCode()) {
				case PType.TYPECODE_BOOLEAN:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getBoolean(), bo2.getBoolean());
					break;
				case PType.TYPECODE_CHAR:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getChar(), bo2.getChar());
					break;
				case PType.TYPECODE_BYTE:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getByte(), bo2.getByte());
					break;
				case PType.TYPECODE_SHORT:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getShort(), bo2.getShort());
					break;
				case PType.TYPECODE_INT:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getInt(), bo2.getInt());
					break;
				case PType.TYPECODE_LONG:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getLong(), bo2.getLong());
					break;
				case PType.TYPECODE_STRING:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getString(), bo2.getString());
					break;
				case PType.TYPECODE_DATE:
					assertEquals(msg + " The operands do not have the same value",
						bo1.getDate(), bo2.getDate());
					break;
				case PType.TYPECODE_FLOAT:
				case PType.TYPECODE_DOUBLE:
					//Uncomparable value
				}
			}
			catch (TypingException e) {
				throw new ExpressionException(e);
			}
		}
		else
			throw new ExpressionException("Expression unknown: " + e1);
	}
}
