package org.objectweb.medor.expression.parser.string;

import org.objectweb.jorm.type.api.PTypeSpace;
import org.objectweb.medor.expression.TestExpressionHelper;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.lib.And;
import org.objectweb.medor.expression.lib.BasicOperand;
import org.objectweb.medor.expression.lib.BasicParameterOperand;
import org.objectweb.medor.expression.lib.DivideBy;
import org.objectweb.medor.expression.lib.Equal;
import org.objectweb.medor.expression.lib.Greater;
import org.objectweb.medor.expression.lib.GreaterEqual;
import org.objectweb.medor.expression.lib.Lower;
import org.objectweb.medor.expression.lib.LowerEqual;
import org.objectweb.medor.expression.lib.Minus;
import org.objectweb.medor.expression.lib.Mod;
import org.objectweb.medor.expression.lib.Mult;
import org.objectweb.medor.expression.lib.NotEqual;
import org.objectweb.medor.expression.lib.Or;
import org.objectweb.medor.expression.lib.Plus;
import org.objectweb.medor.expression.lib.ConditionalAnd;
import org.objectweb.medor.expression.lib.Concat;
import org.objectweb.medor.expression.lib.TypeConverter;
import org.objectweb.medor.expression.parser.api.ParameterTypeProvider;
import org.objectweb.medor.expression.parser.lib.MapParameterTypeProvider;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * @author	P. Dechamboux
 */
public class TestExpression extends TestExpressionHelper {
	public TestExpression(String s) {
		super(s);
	}

	public void testConstStringExtract() {
		try {
			ExpressionParser expr = new ExpressionParser();
			String result;
			ArrayList stringConst = new ArrayList();
			result = expr.extractStringConst("a == 5", stringConst);
			assertEquals("Unexpected string constant extracted", 0, stringConst.size());
			assertEquals("Unexpected result", "a==5", result);
			stringConst.clear();
			result = expr.extractStringConst("a == \"\"", stringConst);
			assertEquals("Bad string constant extracted", "", stringConst.get(0));
			assertEquals("Unexpected result", "a==$", result);
			stringConst.clear();
			result = expr.extractStringConst("a == \"toto\"", stringConst);
			assertEquals("Bad string constant extracted", "toto", stringConst.get(0));
			assertEquals("Unexpected result", "a==$", result);
			stringConst.clear();
			result = expr.extractStringConst("\"toto\" == a", stringConst);
			assertEquals("Bad string constant extracted", "toto", stringConst.get(0));
			assertEquals("Unexpected result", "$==a", result);
			stringConst.clear();
			result = expr.extractStringConst("a == \"toto\"", stringConst);
			assertEquals("Bad string constant extracted", "toto", stringConst.get(0));
			assertEquals("Unexpected result", "a==$", result);
			stringConst.clear();
			result = expr.extractStringConst("\"titi\" == \"toto\"", stringConst);
			assertEquals("Bad string constant extracted",
					Arrays.asList(new String[]{"titi", "toto"}), stringConst);
			assertEquals("Unexpected result", "$==$", result);
			stringConst.clear();
			result = expr.extractStringConst(" a  ==   \"titi\"  + \"toto\"  ", stringConst);
			assertEquals("Bad string constant extracted",
					Arrays.asList(new String[]{"titi", "toto"}), stringConst);
			assertEquals("Unexpected result", "a==$+$", result);
			stringConst.clear();
			result = expr.extractStringConst("(a == \"toto\")", stringConst);
			assertEquals("Bad string constant extracted", "toto", stringConst.get(0));
			assertEquals("Unexpected result", "(a==$)", result);
		} catch (MalformedExpressionException e) {
			fail(e.getMessage());
		}
	}

	public void testAtomicOperand() {
		Map ptmap = new HashMap();
		ptmap.put("a", PTypeSpace.INT);
		ptmap.put("azerty", PTypeSpace.INT);
		ParameterTypeProvider ptp = new MapParameterTypeProvider(ptmap);

		try {
			ExpressionParser parser = new ExpressionParser();
			Expression expr;
			expr = parser.parse("1", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand((byte) 1), expr, logger);
			expr = parser.parse("12", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand((byte) 12), expr, logger);
			expr = parser.parse("\"azerty\"", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand("azerty"), expr, logger);
			expr = parser.parse("\"\"", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand(""), expr, logger);
			expr = parser.parse("\"a\"", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand("a"), expr, logger);
			expr = parser.parse("a", ptp);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicParameterOperand(PTypeSpace.INT, "a"), expr, logger);
			expr = parser.parse("azerty", ptp);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicParameterOperand(PTypeSpace.INT, "azerty"), expr, logger);
			expr = parser.parse("(1)", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand((byte) 1), expr, logger);
			expr = parser.parse("(12)", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand((byte) 12), expr, logger);
			expr = parser.parse("(\"azerty\")", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand("azerty"), expr, logger);
			expr = parser.parse("(\"\")", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand(""), expr, logger);
			expr = parser.parse("(\"a\")", null);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicOperand("a"), expr, logger);
			expr = parser.parse("(a)", ptp);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicParameterOperand(PTypeSpace.INT, "a"), expr, logger);
			expr = parser.parse("(azerty)", ptp);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicParameterOperand(PTypeSpace.INT, "azerty"), expr, logger);
			expr = parser.parse("((azerty))", ptp);
			TestExpressionHelper.equals("Wrong canonical expr", new BasicParameterOperand(PTypeSpace.INT, "azerty"), expr, logger);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testArithmExpr() {
		try {
			ExpressionParser parser = new ExpressionParser();
			Expression expr;
			expr = parser.parse("1 + 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 - 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Minus(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 * 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Mult(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 / 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new DivideBy(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 % 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Mod(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 & 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new And(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 | 2", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Or(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("1 + 2 + 3", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new BasicOperand((byte) 1),
							new Plus(new BasicOperand((byte) 2),
									new BasicOperand((byte) 3))),
					expr, logger);
			expr = parser.parse("1 + 2 - 3", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new BasicOperand((byte) 1),
							new Minus(new BasicOperand((byte) 2),
									new BasicOperand((byte) 3))),
					expr, logger);
			expr = parser.parse("1 * 2 / 3", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Mult(new BasicOperand((byte) 1),
							new DivideBy(new BasicOperand((byte) 2),
									new BasicOperand((byte) 3))),
					expr, logger);
			expr = parser.parse("1 + 2 * 3", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new BasicOperand((byte) 1),
							new Mult(new BasicOperand((byte) 2),
									new BasicOperand((byte) 3))),
					expr, logger);
			expr = parser.parse("1 * 2 + 3", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new Mult(new BasicOperand((byte) 1),
								new BasicOperand((byte) 2)),
							new BasicOperand((byte) 3)),
					expr, logger);
			expr = parser.parse("(1 + 2)", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new BasicOperand((byte) 1), new BasicOperand((byte) 2)),
					expr, logger);
			expr = parser.parse("(1 + 2) + 3", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new Plus(new BasicOperand((byte) 1),
								new BasicOperand((byte) 2)),
							new BasicOperand((byte) 3)),
					expr, logger);
			expr = parser.parse("1 + (2 + 3)", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new BasicOperand((byte) 1),
							new Plus(new BasicOperand((byte) 2),
									new BasicOperand((byte) 3))),
					expr, logger);
			expr = parser.parse("(1 + 2) * (3 + 4)", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Mult(new Plus(new BasicOperand((byte) 1),
								new BasicOperand((byte) 2)),
							new Plus(new BasicOperand((byte) 3),
									new BasicOperand((byte) 4))),
					expr, logger);
			expr = parser.parse("(1 + 2) * 3 + 4", null);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Plus(new Mult(
								new Plus(new BasicOperand((byte) 1),
									new BasicOperand((byte) 2)),
								new BasicOperand((byte) 3)),
							new BasicOperand((byte) 4)),
					expr, logger);

			expr = parser.parse("(\"toto\" + 2)", null);
			TestExpressionHelper.equals("Wrong canonical expr",
				new Concat(new BasicOperand("toto"),
					new TypeConverter(
						new BasicOperand((byte) 2), PTypeSpace.STRING)),
				expr, logger);
			expr = parser.parse("(\"toto\" + \"titi\")", null);
			TestExpressionHelper.equals("Wrong canonical expr",
				new Concat(new BasicOperand("toto"), new BasicOperand("titi")),
				expr, logger);
			expr = parser.parse("((\"toto\" + 2) + 3)", null);
			TestExpressionHelper.equals("Wrong canonical expr",
				new Concat(
					new Concat(new BasicOperand("toto"),
						new TypeConverter(
							new BasicOperand((byte) 2), PTypeSpace.STRING)),
					new TypeConverter(
							new BasicOperand((byte) 3), PTypeSpace.STRING)),
				expr, logger);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}

	public void testCompExpr() {
		try {
			ExpressionParser parser = new ExpressionParser();
			Expression expr;
			Map ptmap = new HashMap();
			ptmap.put("a", PTypeSpace.INT);
			ptmap.put("b", PTypeSpace.INT);
			ptmap.put("c", PTypeSpace.STRING);
			ParameterTypeProvider ptprov =
					new MapParameterTypeProvider(ptmap);
			expr = parser.parse("c == \"toto\"", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Equal(new BasicParameterOperand(PTypeSpace.STRING, "c"),
						new BasicOperand("toto")), expr, logger);
			expr = parser.parse("a == 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Equal(new BasicParameterOperand(PTypeSpace.INT, "a"),
						new BasicOperand((byte) 2)), expr, logger);
			expr = parser.parse("a != 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new NotEqual(new BasicParameterOperand(PTypeSpace.INT, "a"),
						new BasicOperand((byte) 2)), expr, logger);
			expr = parser.parse("a < 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Lower(new BasicParameterOperand(PTypeSpace.INT, "a"),
						new BasicOperand((byte) 2)), expr, logger);
			expr = parser.parse("a <= 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new LowerEqual(new BasicParameterOperand(PTypeSpace.INT, "a"),
						new BasicOperand((byte) 2)), expr, logger);
			expr = parser.parse("a > 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Greater(new BasicParameterOperand(PTypeSpace.INT, "a"),
						new BasicOperand((byte) 2)), expr, logger);
			expr = parser.parse("a >= 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new GreaterEqual(new BasicParameterOperand(PTypeSpace.INT, "a"),
						new BasicOperand((byte) 2)), expr, logger);
			expr = parser.parse("a + 1 == b - 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new Equal(
							new Plus(new BasicParameterOperand(PTypeSpace.INT, "a"),
									new BasicOperand((byte) 1)),
							new Minus(new BasicParameterOperand(PTypeSpace.INT, "b"),
									new BasicOperand((byte) 2))
					),
					expr, logger);
			expr = parser.parse("(a + 1 == b - 2) && a - 4 != b * 2", ptprov);
			TestExpressionHelper.equals("Wrong canonical expr",
					new ConditionalAnd(
						new Equal(
								new Plus(new BasicParameterOperand(PTypeSpace.INT, "a"),
										new BasicOperand((byte) 1)),
								new Minus(new BasicParameterOperand(PTypeSpace.INT, "b"),
										new BasicOperand((byte) 2))
						),
						new NotEqual(
								new Minus(new BasicParameterOperand(PTypeSpace.INT, "a"),
										new BasicOperand((byte) 4)),
								new Mult(new BasicParameterOperand(PTypeSpace.INT, "b"),
										new BasicOperand((byte) 2))
						)),
					expr, logger);
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}
}
